@@ -1,4 +1,4 @@
-# Copyright 2008-2018 Kouhei Sutou <kou@cozmixng.org>
+# Copyright 2008-2019 Kouhei Sutou <kou@cozmixng.org>
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -34,6 +34,11 @@ class PackageConfig
       @native_pkg_config ||= guess_native_pkg_config
     end
 
+    @native_pkg_config_prefix = nil
+    def native_pkg_config_prefix
+      @native_pkg_config_prefix ||= compute_native_pkg_config_prefix
+    end
+
     @custom_override_variables = nil
     def custom_override_variables
       @custom_override_variables ||= with_config("override-variables", "")
@@ -41,6 +46,7 @@ class PackageConfig
 
     def clear_configure_args_cache
       @native_pkg_config = nil
+      @native_pkg_config_prefix = nil
       @custom_override_variables = nil
     end
 
@@ -54,10 +60,12 @@ class PackageConfig
     end
 
     def guess_native_pkg_config
-      pkg_config = with_config("pkg-config", ENV["PKG_CONFIG"] || "pkg-config")
+      exeext = RbConfig::CONFIG["EXEEXT"]
+      default_pkg_config = ENV["PKG_CONFIG"] || "pkg-config#{exeext}"
+      pkg_config = with_config("pkg-config", default_pkg_config)
       pkg_config = Pathname.new(pkg_config)
       unless pkg_config.absolute?
-        found_pkg_config = search_pkg_config_from_path(pkg_config)
+        found_pkg_config = search_executable_from_path(pkg_config)
         pkg_config = found_pkg_config if found_pkg_config
       end
       unless pkg_config.absolute?
@@ -67,10 +75,10 @@ class PackageConfig
       pkg_config
     end
 
-    def search_pkg_config_from_path(pkg_config)
+    def search_executable_from_path(name)
       (ENV["PATH"] || "").split(SEPARATOR).each do |path|
-        try_pkg_config = Pathname(path) + pkg_config
-        return try_pkg_config if try_pkg_config.exist?
+        try_name = Pathname(path) + name
+        return try_name if try_name.executable?
       end
       nil
     end
@@ -110,6 +118,20 @@ class PackageConfig
         Pathname(path.to_s)
       end
     end
+
+    def compute_native_pkg_config_prefix
+      pkg_config = native_pkg_config
+      return nil unless pkg_config.absolute?
+
+      pkg_config_prefix = pkg_config.parent.parent
+      if File::ALT_SEPARATOR
+        normalized_pkg_config_prefix =
+          pkg_config_prefix.to_s.split(File::ALT_SEPARATOR).join(File::SEPARATOR)
+        Pathname(normalized_pkg_config_prefix)
+      else
+        pkg_config_prefix
+      end
+    end
   end
 
   attr_reader :paths
@@ -230,20 +252,18 @@ class PackageConfig
   end
 
   def normalize_path_flags(path_flags, flag_option)
+    return path_flags unless /-mingw32\z/ === RUBY_PLATFORM
+
+    pkg_config_prefix = self.class.native_pkg_config_prefix
+    return path_flags unless pkg_config_prefix
+
+    mingw_dir = pkg_config_prefix.basename.to_s
     path_flags.collect do |path_flag|
-      path = path_flag.sub(flag_option, "")
-      prefix = ""
-      case RUBY_PLATFORM
-      when "x86-mingw32"
-        prefix = Dir.glob("c:/msys{32,64,*}").first
-      when "x64-mingw32"
-        prefix = Dir.glob("c:/msys{64,*}").first
-      end
-      if /\A[a-z]:/i === path
-        "#{flag_option}#{path}"
-      else
-        "#{flag_option}#{prefix}#{path}"
+      path = path_flag.sub(/\A#{Regexp.escape(flag_option)}/, "")
+      path = path.sub(/\A\/#{Regexp.escape(mingw_dir)}/i) do
+        pkg_config_prefix.to_s
       end
+      "#{flag_option}#{path}"
     end
   end
 
@@ -355,39 +375,40 @@ class PackageConfig
       "/opt/X11/lib/pkgconfig",
       "/usr/share/pkgconfig",
     ]
-    case RUBY_PLATFORM
-    when "x86-mingw32"
-      default_paths.concat(Dir.glob("c:/msys*/mingw32/lib/pkgconfig"))
-    when "x64-mingw32"
-      default_paths.concat(Dir.glob("c:/msys*/mingw64/lib/pkgconfig"))
-    end
     libdir = ENV["PKG_CONFIG_LIBDIR"]
     default_paths.unshift(libdir) if libdir
 
-    pkg_config = self.class.native_pkg_config
-    return default_paths.join(SEPARATOR) unless pkg_config.absolute?
-
-    pkg_config_prefix = pkg_config.parent.parent
-    pkg_config_arch_depended_paths =
-      Dir.glob((pkg_config_prefix + "lib/*/pkgconfig").to_s)
     paths = []
-    paths.concat(pkg_config_arch_depended_paths)
-    paths << (pkg_config_prefix + "lib64/pkgconfig").to_s
-    paths << (pkg_config_prefix + "libx32/pkgconfig").to_s
-    paths << (pkg_config_prefix + "lib/pkgconfig").to_s
-    paths << (pkg_config_prefix + "libdata/pkgconfig").to_s
+    pkg_config_prefix = self.class.native_pkg_config_prefix
+    if pkg_config_prefix
+      pkg_config_arch_depended_paths =
+        Dir.glob((pkg_config_prefix + "lib/*/pkgconfig").to_s)
+      paths.concat(pkg_config_arch_depended_paths)
+      paths << (pkg_config_prefix + "lib64/pkgconfig").to_s
+      paths << (pkg_config_prefix + "libx32/pkgconfig").to_s
+      paths << (pkg_config_prefix + "lib/pkgconfig").to_s
+      paths << (pkg_config_prefix + "libdata/pkgconfig").to_s
+    end
     if /-darwin\d[\d\.]*\z/ =~ RUBY_PLATFORM and
         /\A(\d+\.\d+)/ =~ `sw_vers -productVersion`
       mac_os_version = $1
       homebrew_repository_candidates = []
-      brew_path = pkg_config_prefix + "bin" + "brew"
-      if brew_path.exist?
-        escaped_brew_path = Shellwords.escape(brew_path.to_s)
-        homebrew_repository = `#{escaped_brew_path} --repository`.chomp
-        homebrew_repository_candidates << Pathname.new(homebrew_repository)
+      if pkg_config_prefix
+        brew_path = pkg_config_prefix + "bin" + "brew"
+        if brew_path.exist?
+          escaped_brew_path = Shellwords.escape(brew_path.to_s)
+          homebrew_repository = `#{escaped_brew_path} --repository`.chomp
+          homebrew_repository_candidates << Pathname.new(homebrew_repository)
+        else
+          homebrew_repository_candidates << pkg_config_prefix + "Homebrew"
+          homebrew_repository_candidates << pkg_config_prefix
+        end
       else
-        homebrew_repository_candidates << pkg_config_prefix + "Homebrew"
-        homebrew_repository_candidates << pkg_config_prefix
+        brew = self.class.__send__(:search_executable_from_path, "brew")
+        if brew
+          homebrew_repository = `brew --repository`.chomp
+          homebrew_repository_candidates << Pathname(homebrew_repository)
+        end
       end
       homebrew_repository_candidates.each do |candidate|
         path = candidate + "Library/Homebrew/os/mac/pkgconfig/#{mac_os_version}"
